"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from cfn_policy_validator import ApplicationError
from cfn_policy_validator.cfn_tools.schema_validator import validate_schema


class AndEvaluator:
	def __init__(self, node_evaluator):
		self.node_evaluator = node_evaluator

	def evaluate(self, value, visited_nodes):
		validate_schema(value, and_schema, 'Fn::And')

		result = True
		for index, and_candidate in enumerate(value):
			evaluated_value = self.node_evaluator.eval(and_candidate, visited_nodes)
			if not isinstance(evaluated_value, bool):
				raise ApplicationError('Could not evaluate Fn::And. All values of an AND function must be booleans. '
									   f'Value at index {index} does not evaluate to a boolean: {value}')

			result = result and evaluated_value

		return result


and_schema = {
	'type': 'array',
	'items': [
		{}
	],
	'minItems': 2,
	'maxItems': 10
}



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from cfn_policy_validator import ApplicationError
from cfn_policy_validator.cfn_tools.schema_validator import validate_schema


class NotEvaluator:
	def __init__(self, node_evaluator):
		self.node_evaluator = node_evaluator

	def evaluate(self, value, visited_nodes):
		validate_schema(value, not_schema, 'Fn::Not')

		evaluated = self.node_evaluator.eval(value[0], visited_nodes=visited_nodes)
		if not isinstance(evaluated, bool):
			raise ApplicationError('Could not evaluate Fn::Not. The evaluated value of a Not function must be a boolean. '
								   f'Value does not evaluate to a boolean: {value}')

		return not evaluated


not_schema = {
	'type': 'array',
	'items': [
		{}
	],
	'minItems': 1,
	'maxItems': 1
}



from cfn_policy_validator import ApplicationError
from cfn_policy_validator.cfn_tools.schema_validator import validate_schema


class ConditionsEvaluator:
	def __init__(self, conditions, node_evaluator):
		self.conditions = conditions
		self.node_evaluator = node_evaluator

	def eval(self, condition_name, visited_nodes: list = None) -> bool:
		visited_nodes = [] if visited_nodes is None else visited_nodes

		intrinsic_function = self.conditions.get(condition_name)
		if intrinsic_function is None:
			raise ApplicationError(f'Unable to find referenced condition in template: {condition_name}')

		validate_schema(intrinsic_function, condition_schema, f'Condition.{condition_name}')

		# there are caveats here if the condition includes a REF, we won't know the exact value until deployment
		# TODO: maybe add a warning finding?
		return self.node_evaluator.eval(intrinsic_function, visited_nodes=visited_nodes)


condition_schema = {
	'type': 'object'
}



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from cfn_policy_validator.cfn_tools.schema_validator import validate_schema


class IfEvaluator:
	def __init__(self, condition_evaluator, node_evaluator):
		self.node_evaluator = node_evaluator
		self.condition_evaluator = condition_evaluator

	def evaluate(self, value, visited_nodes):
		validate_schema(value, if_schema, 'Fn::If')

		condition_name = value[0]
		value_if_true = value[1]
		value_if_false = value[2]

		condition_is_true = self.condition_evaluator.eval(condition_name, visited_nodes)
		if condition_is_true:
			return self.node_evaluator.eval(value_if_true, visited_nodes=visited_nodes)
		else:
			return self.node_evaluator.eval(value_if_false, visited_nodes=visited_nodes)


if_schema = {
	'type': 'array',
	'items': [
		{
			'type': 'string'
		},
		{},
		{}
	],
	'minItems': 3,
	'maxItems': 3
}



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from cfn_policy_validator.cfn_tools.schema_validator import validate_schema


class EqualsEvaluator:
	def __init__(self, node_evaluator):
		self.node_evaluator = node_evaluator

	def evaluate(self, value, visited_nodes):
		validate_schema(value, equals_schema, 'Fn::Equals')

		first_value = value[0]
		second_value = value[1]

		first_value_evaluated = self.node_evaluator.eval(first_value, visited_nodes=visited_nodes)
		second_value_evaluated = self.node_evaluator.eval(second_value, visited_nodes=visited_nodes)

		return str(first_value_evaluated) == str(second_value_evaluated)


equals_schema = {
	'type': 'array',
	'minItems': 2,
	'maxItems': 2,
	'items': [
		{},
		{}
	],
	'additionalItems': False
}



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from cfn_policy_validator.cfn_tools.schema_validator import validate_schema


class ConditionFunctionEvaluator:
	def __init__(self, condition_evaluator):
		self.condition_evaluator = condition_evaluator

	def evaluate(self, value, visited_nodes):
		validate_schema(value, condition_function_schema, 'Condition')
		return self.condition_evaluator.eval(value, visited_nodes)


condition_function_schema = {
	'type': 'string',
	'additionalItems': False
}
